<?php
/*
Plugin Name: Gosh Gallery
Plugin URI: http://www.gosh.live/
Description: Find content created by your fans! Track, gather and publish content on one platform using #hashtags or @usernames.
Version: 1.0.0
Author: Tapastreet Ltd.
Author URI: http://www.gosh.live/
License: GPLv2 or later
*/
defined('ABSPATH') or die('Plugin file cannot be accessed directly.');

define('GOSH_MESSAGES_VAR',		    '_gosh_gallery_messages');
define('GOSH_POST_TYPE',   			'gosh_gallery');
define('GOSH_PLUGIN_DIR', 			'gosh');
define('GOSH_SHORTCODE', 			'gosh_gallery');
define('GOSH_PLUGIN_ROOT', 			dirname(__FILE__));
define('GOSH_PLUGIN_URL',  			plugins_url());

/* Main configuration
 * CHECK BEFORE DEPLOYMENT
 * */
define('GOSH_APP_NAME',               'Social Gallery');
define('GOSH_VERSION', 				  '1.0.0');
define('GOSH_API_ORIGIN',   		  'https://wordpress-api.tapastreet.com');
define('GOSH_API_AUTH_ROOT',          GOSH_API_ORIGIN.'/auth/wordpress');
define('GOSH_API_VIEWER_ROOT',        'https://widget-viewer.tapastreet.com/');
define('GOSH_API_URL',				  'https://www.gosh.live/dashboard/process/api.php');
define('GOSH_VIEWER_ASSETS',          'https://viewer-assets.tapastreet.com/');
define('GOSH_REFRESH_INTERVAL',  	  300000);
define('GOSH_AUTOPLAY_INTERVAL',	  5000);
/* END Main configuration */
define('GOSH_APP_TOKEN',              get_option('gosh_app_token'));
define('GOSH_USER_TOKEN',    		  get_option('gosh_user_token'));
define('GOSH_SITE_ID',				  get_option('gosh_site_id'));
define('GOSH_USER_ID',     		      get_option('gosh_user_id'));
define('GOSH_SETTINGS_VAR',			  "gosh_settings");

include GOSH_PLUGIN_ROOT . '/src/Gosh.php';

/* Hooks for initialization and removal
 * */
function gosh_plugin_activation(){
	// Create custom user and site ID's on plugin activation
	update_option ('gosh_app_token',   '+Yg55d,s~5>?X"~w');
	update_option ('gosh_user_token',  '');
	update_option ('gosh_site_id',     uniqid('wps'));
	update_option ('gosh_user_id',     uniqid('wpu'));
}
register_activation_hook(__FILE__, 'gosh_plugin_activation'); // Names of functions should have unique prefix (gosh_)

function gosh_plugin_deactivation() {
	// Cleanup after plugin deactivation
	// Remove all posts
	$gosh_posts = get_pages( array( 'post_type' => GOSH_POST_TYPE, 'number' => 50) );
    foreach( $gosh_posts as $post ) {
      wp_delete_post( $post->ID, true); // true = delete, false = trash
	}
	// Remove plugin options | meta data is removed automatically
    delete_option('gosh_app_token');
	delete_option('gosh_user_token');
	delete_option('gosh_site_id');
	delete_option('gosh_user_id');
}
register_deactivation_hook(__FILE__, 'gosh_plugin_deactivation');

/* Insert scripts
 */
add_action('wp_enqueue_scripts', 'gosh_scripts');
function gosh_scripts() {
	 // enqueue built-in script
     wp_enqueue_script('jquery');
	 // enqueue custom script(s)
     wp_register_script('gosh_init', plugins_url('js/gosh_init.js', __FILE__), array('jquery')); // Third parameter specifies the dependent libraries.
     // Now we can include scripts
     wp_enqueue_script('gosh_init');
}

/* Insert styles
 */
add_action('wp_enqueue_scripts',    'gosh_styles');
add_action('admin_enqueue_scripts', 'gosh_styles' );
function gosh_styles() {
  wp_register_style('gosh_main', plugins_url('css/style.css', __FILE__));
  wp_enqueue_style('gosh_main');
}

/* Add GOSH Gallery post type
 */
add_action('init', 'gosh_register_plugin');
function gosh_register_plugin() {
    $args = array(
       'labels' => array('menu_name' => _x('Social Gallery', GOSH_POST_TYPE)),
       'hierarchical' => true,
       'description' => 'Gosh social galleries',
       'supports' => array('title'),
       'public' => true,
       'show_ui' => true,
       'show_in_menu' => true,
       'show_in_nav_menus' => true,
       'publicly_queryable' => false, // permalink
       'exclude_from_search' => true,
       'has_archive' => false, // Preview button
       'query_var' => true,
       'can_export' => true,
       'rewrite' => true,
       'menu_icon' => GOSH_PLUGIN_URL.'/'.GOSH_PLUGIN_DIR.'/img/icon_20x20.png',
       'capability_type' => 'post'
    );
    register_post_type(GOSH_POST_TYPE, $args);
}

/*
 * Gallery shortcode
 * */

// We can pass any number of attributes to shortcode and you have to include them inside the extract function to retrieve the values.
add_shortcode(GOSH_SHORTCODE, "gosh_display_gallery");
function gosh_display_gallery($attr, $content) {
	global $gosh;
	$id = 0;
    extract(shortcode_atts(array('id' => ''), $attr));
	$settings = $gosh->getSettings($id);
	$html     = $gosh->renderTemplate('gallery', $settings);
	return $html;
}

// Enables parsing the shortcode inside text widget
add_filter('widget_text', 'do_shortcode');

/* Saving post
 */
add_action( 'admin_footer', 'gosh_save_settings_ajax' );
function gosh_save_settings_ajax() {
	global $post;
	?>
	<script type="text/javascript">
	function GoshSaveSettingsAjax(token, hash_url, callback){
		var hash_url = hash_url || 'placeholder_<?php echo $post->ID; ?>';
		var data = {
			'gosh_hash_url'  	   : hash_url,
			'action'         	   : 'save_settings',
			'post_id'        	   : <?php if($post->ID) echo $post->ID; else echo "0"; ?>,
			'post_type'      	   : '<?php echo GOSH_POST_TYPE; ?>',
			'gosh_nonce'     	   : '<?php echo wp_create_nonce('gosh_nonce'); ?>',
			'gosh_token'       	   : token
		};
		// ajaxurl is always defined in the admin header and points to admin-ajax.php
		jQuery.post(ajaxurl, data, function(response) {
			if(callback) callback(response);
		});
	};
	</script>
	<?php
}

add_action( 'wp_ajax_save_settings', 'gosh_save_settings_callback' );
function gosh_save_settings_callback() {
	//global $wpdb; // this is how you get access to the database
	echo gosh_save_settings($_POST['post_id']);
	wp_die(); // this is required to terminate immediately and return a proper response
}

add_action('save_post', 'gosh_save_settings');
function gosh_save_settings($post_id) {
	global $gosh;
    // verify nonce
    if (!wp_verify_nonce($_POST['gosh_nonce'], 'gosh_nonce')) { return "noonce".$post_id; }
    // check autosave
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) { return "autosave".$post_id; }
    // check permissions
    if (GOSH_POST_TYPE === $_POST['post_type'] && current_user_can('edit_post', $post_id)) {
		// Prepare token parameter
		$token = sanitize_text_field($_POST['gosh_token']); // Sanitized
		$token = strlen($token) === 15 ? $token : "";  // Validated
		if ($token && !get_option('gosh_user_token')) {
			update_option('gosh_user_token', $token);  // Update database
		}
		// Prepare settings parameter
		$hash_url = sanitize_text_field($_POST['gosh_hash_url']); // Sanitized
		if (strlen($hash_url) < 10 || strlen($hash_url) > 60) { // Validated
			$hash_url = "";
		}
		$data = array("hash_url" => $hash_url, "post_id" => intval($post_id));
		// Validate key_word
		$result = $gosh->validateSettings(array("key_word" => $_POST['gosh_key_word']));
		if ($result !== false) {
			//exit('Saving  settings');
			$gosh->saveSettings($post_id, $data); // Update database
			return json_encode(array("status" => "success", "message" => "Settings saved."));
		} else {
			$gosh->addMessage($post_id, 'Invalid <strong>#hashtag</strong> or <strong>@username</strong>', 'error');
			return json_encode(array("status" => "error", "message" => "Invalid input."));
		}
	}
}

/* Create meta-box content (settings) for Gosh type post
*/
add_action('add_meta_boxes', 'gosh_config_content');
function gosh_config_content() {
    add_meta_box('gosh-gallery-config', 'Settings', 'gosh_view_settings', GOSH_POST_TYPE, 'normal');
}
function gosh_view_settings() {
   global $gosh;
   global $post;
   // Setting page
   $settingsUI = $gosh->renderTemplate('settings', $gosh->getSettings($post->ID));
   $gosh->printMessages($post->ID);
   echo $settingsUI;
}
$gosh = new Gosh;